//MyRenderer.cpp
#include <d3d9.h>
#include <d3dx9.h>

#include "MyRenderer.h"
#include "MyMesh.h"

#include "MyVertex.h"
#include "MyVertexBuffer.h"
#include "MyIndexBuffer.h"

//------------------------------------------------------------------------------------------
MyRenderer::MyRenderer()
{
	
}
//------------------------------------------------------------------------------------------
MyRenderer::~MyRenderer()
{

}
//------------------------------------------------------------------------------------------
bool MyRenderer::Update()
{
	return true;
}
//------------------------------------------------------------------------------------------
bool MyRenderer::Render(IDirect3DDevice9 *pkDevice, MyMesh& mesh)
{
	if(mesh.GetDeviceVertex() == 0)
	{
		MakeVertexBuffer(pkDevice, mesh);
	}

	MyVertexBuffer* pVertexBuffer = (MyVertexBuffer *)mesh.GetDeviceVertex();
	pVertexBuffer->Render(pkDevice, mesh.GetPrimitiveNumber(), D3DPT_TRIANGLELIST);
	
	return true;
}
//------------------------------------------------------------------------------------------
DWORD MyRenderer::SeekVertexFVF(MyMesh& mesh)
{
	DWORD fvf = 0;
	if(mesh.GetVertexData())
		fvf |= D3DFVF_XYZ;
	if(mesh.GetVertexColor())
		fvf |= D3DFVF_DIFFUSE;
	if(mesh.GetTextureUV())
		fvf |= D3DFVF_TEX1;

	return fvf;
}
//------------------------------------------------------------------------------------------
int MyRenderer::SeekBufferSize(DWORD fvf)
{
	int iSize = 0;

	if(fvf & D3DFVF_XYZ)
		iSize += (3 * sizeof(float));
	if(fvf & D3DFVF_DIFFUSE)
		iSize += (sizeof(DWORD));
	if(fvf & D3DFVF_TEX1)
		iSize += (2 * sizeof(float));
	return iSize;
}
//------------------------------------------------------------------------------------------
unsigned char* MyRenderer::AllocVertexBuffer(int iVertexCount, int iVertexSize, MyMesh& mesh)
{
	int iSize = iVertexCount * iVertexSize;
	unsigned char* pBuff = new unsigned char [iSize];

	MyVector3* pVertex = mesh.GetVertexData();
	DWORD*	  pColor = mesh.GetVertexColor();
	MyVector2* pUV = mesh.GetTextureUV();

	unsigned char* p = pBuff;
	for(int i = 0; i < iVertexCount; ++i)
	{
		if(pVertex)
		{
			memcpy(p, pVertex, 3 * sizeof(float));
			p += 3 * sizeof(float);
			pVertex++;
		}
		if(pColor)
		{
			memcpy(p, pColor, sizeof(DWORD));
			p += sizeof(DWORD);
			pColor++;
		}
		if(pUV)
		{
			memcpy(p, pUV, 2 * sizeof(float));
			p += 2 * sizeof(float);
			pUV++;
		}
	}

	return pBuff;
}
//------------------------------------------------------------------------------------------
bool MyRenderer::MakeVertexBuffer(IDirect3DDevice9 *pkDevice, MyMesh& mesh)
{
	int iVertexCount = mesh.GetVertexNumber(); 
	DWORD fvf = SeekVertexFVF(mesh);
	int	  iBufferSize = SeekBufferSize(fvf);

	MyVertexBuffer* pVertexBuffer = new MyVertexBuffer;
	pVertexBuffer->CreateBuffer(pkDevice, iVertexCount, fvf, iBufferSize);
	
	unsigned char* pBuffer = AllocVertexBuffer(iVertexCount, iBufferSize, mesh);
	pVertexBuffer->SetData((void*)pBuffer);

	delete [] pBuffer;

	//ε  
	USHORT* pIndex = mesh.GetVertexIndex();
	if(pIndex)
	{
		int pIndexCount = mesh.GetVertexIndexCount();
		MyIndexBuffer* pIndexBuffer = new MyIndexBuffer;
		pIndexBuffer->CreateBuffer( pkDevice, pIndexCount, D3DFMT_INDEX16 );
		pIndexBuffer->SetData(pIndexCount, pIndex);
		pVertexBuffer->SetIndexBuffer( pIndexBuffer );
	}
	mesh.SetDeviceVertex(pVertexBuffer);
	return true;
}
//------------------------------------------------------------------------------------------